@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2023 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -22,6 +22,12 @@
 #include "org_postgresql_pljava_internal_AclId.h"
 #include "pljava/Exception.h"
 
+#if PG_VERSION_NUM >= 160000
+#include <catalog/pg_namespace.h>
+#define pg_namespace_aclcheck(oid,rid,mode) \
+	object_aclcheck(NamespaceRelationId, (oid), (rid), (mode))
+#endif
+
 static jclass    s_AclId_class;
 static jmethodID s_AclId_init;
 static jfieldID  s_AclId_m_native;
@@ -184,11 +190,7 @@ Java_org_postgresql_pljava_internal_AclId__1getName(JNIEnv* env, jobject aclId)
 	{
 		result = String_createJavaStringFromNTS(
 			GetUserNameFromId(
-#if PG_VERSION_NUM >= 90500
 				AclId_getAclId(aclId), /* noerr= */ false
-#else
-				AclId_getAclId(aclId)
-#endif
 			)
 		);
 	}
