@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2021 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2016-2023 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -55,6 +55,7 @@ import java.security.ProtectionDomain;
 import java.sql.ResultSet;
 import java.sql.SQLData;
 import java.sql.SQLException;
+import java.sql.SQLFeatureNotSupportedException;
 import java.sql.SQLInput;
 import java.sql.SQLOutput;
 import java.sql.SQLNonTransientException;
@@ -1318,6 +1319,18 @@ public class Function
 	{
 		Matcher info = parse(procTup);
 
+		/*
+		 * Reject any TRANSFORM FOR TYPE clause at validation time, on
+		 * the grounds that it will get ignored at invocation time anyway.
+		 * The check could be made unconditional, and so catch at invocation
+		 * time any function that might have been declared before this validator
+		 * check was added. But simply ignoring the clause at invocation time
+		 * (as promised...) keeps that path leaner.
+		 */
+		if ( forValidator  &&  null != procTup.getObject("protrftypes") )
+			throw new SQLFeatureNotSupportedException(
+				"a PL/Java function will not apply TRANSFORM FOR TYPE","0A000");
+
 		if ( forValidator  &&  ! checkBody )
 			return null;
 
@@ -1826,7 +1839,7 @@ public class Function
 	 * Uncompiled pattern to recognize a Java identifier.
 	 */
 	private static final String javaIdentifier = String.format(
-		"\\p{%1$sStart}\\p{%1sPart}++", "javaJavaIdentifier"
+		"\\p{%1$sStart}\\p{%1sPart}*+", "javaJavaIdentifier"
 	);
 
 	/**
