@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -31,7 +31,7 @@ import java.util.Map;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.Reader;
-import java.io.UnsupportedEncodingException;
+import static java.nio.charset.StandardCharsets.US_ASCII;
 
 
 /**
@@ -124,9 +124,8 @@ public abstract class ObjectResultSet extends AbstractResultSet
 
 	/**
 	 * Throws "unsupported" exception.
-	 * @deprecated
 	 */
-	@Override
+	@SuppressWarnings("deprecation") @Override
 	public BigDecimal getBigDecimal(int columnIndex, int scale)
 	throws SQLException
 	{
@@ -372,8 +371,8 @@ public abstract class ObjectResultSet extends AbstractResultSet
 
 	/**
 	 * Throws "unsupported" exception.
-	 * @deprecated
 	 */
+	@SuppressWarnings("deprecation") @Override
 	public InputStream getUnicodeStream(int columnIndex)
 	throws SQLException
 	{
@@ -421,16 +420,8 @@ public abstract class ObjectResultSet extends AbstractResultSet
 	public void updateAsciiStream(int columnIndex, InputStream x, int length)
 	throws SQLException
 	{
-		try
-		{
-			updateObject(columnIndex,
-				new ClobValue(new InputStreamReader(x, "US-ASCII"), length));
-		}
-		catch(UnsupportedEncodingException e)
-		{
-			throw new SQLException(
-				"US-ASCII encoding is not supported by this JVM");
-		}
+		updateObject(columnIndex,
+			new ClobValue(new InputStreamReader(x, US_ASCII), length));
 	}
 
 	/**
@@ -628,13 +619,13 @@ public abstract class ObjectResultSet extends AbstractResultSet
 	// ************************************************************
 	// JDBC 4.1
 	// Getter-by-columnIndex
-	// Add @Override here once Java back horizon advances to 7.
 	// ************************************************************
 
 	/**
 	 * Implemented over {@link #getObjectValue(int,Class) getObjectValue}.
 	 * Final because it records {@code wasNull} for use by other methods.
 	 */
+	@Override
 	public final <T> T getObject(int columnIndex, Class<T> type)
 	throws SQLException
 	{
