@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -7,7 +7,7 @@
  * http://opensource.org/licenses/BSD-3-Clause
  *
  * Contributors:
- *   Thomas Hallgren
+ *   Tada AB
  *   PostgreSQL Global Development Group
  *   Chapman Flack
  */
@@ -89,7 +89,7 @@ public class SPIConnection implements Connection
 	 * {@link PreparedStatement#setObject(int,Object)}.
 	 */
 	private static final HashMap<Class<?>,Integer> s_class2sqlType =
-		new HashMap<Class<?>,Integer>(30);
+		new HashMap<>(30);
 
 	static
 	{
@@ -139,8 +139,9 @@ public class SPIConnection implements Connection
 	}
 
 	/**
-	 * Returns a default connection instance. It is the callers responsability
-	 * to close this instance.
+	 * Returns a default connection instance. It is normally the caller's
+	 * responsibility to close this instance, but as {@code close} is a no-op
+	 * for this connection, that isn't critical.
 	 */
 	public static Connection getDefault()
 	throws SQLException
@@ -827,6 +828,7 @@ public class SPIConnection implements Connection
 	 * in {@code ResultSet}s seems a bit suspect, as does its use in UDT input
 	 * but not output with composites.
 	 */
+	@SuppressWarnings("unchecked")
 	static <T> T basicCoercion(Class<T> cls, Object value)
 	throws SQLException
 	{
@@ -887,7 +889,7 @@ public class SPIConnection implements Connection
 				return Long.valueOf((String)value);
 
 			if(value instanceof Boolean)
-				return new Long(((Boolean)value).booleanValue() ? 1 : 0);
+				return ((Boolean)value) ? 1 : 0;
 		}
 		else if(cls == BigDecimal.class)
 		{
@@ -903,7 +905,7 @@ public class SPIConnection implements Connection
 				return Double.valueOf((String)value);
 
 			if(value instanceof Boolean)
-				return new Double(((Boolean)value).booleanValue() ? 1 : 0);
+				return ((Boolean)value) ? 1 : 0;
 		}
 		throw new SQLException("Cannot derive a Number from an object of class " + value.getClass().getName());
 	}
@@ -921,6 +923,7 @@ public class SPIConnection implements Connection
 	 * the use of the same coercion in both the retrieval and storage direction
 	 * in {@code ResultSet}s seems a bit suspect.
 	 */
+	@SuppressWarnings("unchecked")
 	static <T> T basicCalendricalCoercion(
 		Class<T> cls, Object value, Calendar cal)
 	throws SQLException
@@ -1034,29 +1037,6 @@ public class SPIConnection implements Connection
 
 	static
 	{
-		/*
-		 * Try to get the JDBC 4.2 / Java 8 TIME*ZONE types reflectively.
-		 * Once the Java back horizon advances to 8, just do this the easy way.
-		 */
-		int sqx  = Types.OTHER;      // don't just start saying SQLXML in 1.5.1
-		int ttz  = Types.TIME;       // Use these values
-		int tstz = Types.TIMESTAMP;  //         pre-Java 8
-//		try    COMMENTED OUT FOR BACK-COMPATIBILITY REASONS IN PL/JAVA 1.5.x
-//		{
-//			ttz =
-//				Types.class.getField("TIME_WITH_TIMEZONE")
-//					.getInt(Types.class);
-//			tstz =
-//				Types.class.getField("TIMESTAMP_WITH_TIMEZONE")
-//					.getInt(Types.class);
-//		}
-//		catch ( NoSuchFieldException nsfe ) { } // ok, not running in Java 8
-//		catch ( IllegalAccessException iae )
-//		{
-//			throw new ExceptionInInitializerError(iae);
-//		}
-//		sqx = Types.SQLXML;
-
 		JDBC_TYPE_NUMBERS = new int[]
 		{
 			Types.SMALLINT,
@@ -1072,9 +1052,9 @@ public class SPIConnection implements Connection
 			Types.BOOLEAN,
 			Types.BIT,
 			Types.DATE,
-			Types.TIME, ttz,
-			Types.TIMESTAMP, Types.TIMESTAMP, tstz,
-			sqx,
+			Types.TIME, Types.TIME_WITH_TIMEZONE,
+			Types.TIMESTAMP, Types.TIMESTAMP, Types.TIMESTAMP_WITH_TIMEZONE,
+			Types.SQLXML,
 			Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY,
 			Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY,
 			Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY, Types.ARRAY,
@@ -1124,11 +1104,13 @@ public class SPIConnection implements Connection
 	 */
 
 	@Override
-	public void setClientInfo(String name, String value) throws SQLClientInfoException
+	public void setClientInfo(String name, String value)
+	throws SQLClientInfoException
 	{
-		Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+		Map<String, ClientInfoStatus> failures = new HashMap<>();
 		failures.put(name, ClientInfoStatus.REASON_UNKNOWN_PROPERTY);
-		throw new SQLClientInfoException("ClientInfo property not supported.", failures);
+		throw new SQLClientInfoException(
+			"ClientInfo property not supported.", failures);
 	}
 
 
@@ -1139,13 +1121,14 @@ public class SPIConnection implements Connection
 		if (properties == null || properties.size() == 0)
 			return;
 
-		Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+		Map<String, ClientInfoStatus> failures = new HashMap<>();
 
 		Iterator<String> i = properties.stringPropertyNames().iterator();
 		while (i.hasNext()) {
 			failures.put(i.next(), ClientInfoStatus.REASON_UNKNOWN_PROPERTY);
 		}
-		throw new SQLClientInfoException("ClientInfo property not supported.", failures);
+		throw new SQLClientInfoException(
+			"ClientInfo property not supported.", failures);
 	}
 
 	@Override
@@ -1218,21 +1201,21 @@ public class SPIConnection implements Connection
 	// Non-implementation of JDBC 4.1 methods.
 	// ************************************************************
 
-	// add @Override once Java 7 is back-support limit
+	@Override
 	public void abort(Executor executor) throws SQLException
 	{
 		throw new SQLFeatureNotSupportedException(
 			"SPIConnection.abort(Executor) not implemented yet.", "0A000" );
 	}
 
-	// add @Override once Java 7 is back-support limit
+	@Override
 	public int getNetworkTimeout() throws SQLException
 	{
 		throw new SQLFeatureNotSupportedException(
 			"SPIConnection.getNetworkTimeout() not implemented yet.", "0A000" );
 	}
 
-	// add @Override once Java 7 is back-support limit
+	@Override
 	public void setNetworkTimeout(Executor executor, int milliseconds)
 		throws SQLException
 	{
@@ -1240,14 +1223,14 @@ public class SPIConnection implements Connection
 			"SPIConnection.setNetworkTimeout(Executor,int) not implemented yet.", "0A000" );
 	}
 
-	// add @Override once Java 7 is back-support limit
+	@Override
 	public String getSchema() throws SQLException
 	{
 		throw new SQLFeatureNotSupportedException(
 			"SPIConnection.getSchema() not implemented yet.", "0A000" );
 	}
 
-	// add @Override once Java 7 is back-support limit
+	@Override
 	public void setSchema(String schema) throws SQLException
 	{
 		throw new SQLFeatureNotSupportedException(
